//************************************************************
//
// modulo:      main.c
// autore:      Luciano Bigozzi
// project:     sinottico
// mcu:         PIC16F887
// clock:       esterno @ 20MHz
// board:       
// compilatore: Microchip XC8 (2.05)
// ide:         MPLAB X IDE (5.30)
// programm.:   PICkit 4
//
//************************************************************

/* ATTENZIONE !
 * dato che il PICkit comunica su RB6 e RB7, per questo esempio non  possibile
 * utilizzare il PICkit come sorgente di alimentazione altrimenti interferisce
 * con i pulsanti 3 e 4. In alternativa se avete realizzato il circuito su 
 * millefori, potete collegare i pulsanti su altre porte ma ricordate che c' 
 * bisogno delle resistenze di pull-up esterne se il PICmicro che avete scelto
 * non le ha sulle porte su cui avete collegato i pulsanti. Il PIC16F887 ha le
 * resistenze di pull-up solo sul banco B. Il 16F18877 le ha su tutti i banchi,
 * su quest'ultimo si possono abilitare agendo sui registri WPUA, WPUB, WPUC,
 * WPUD,WPUE (dopo aver disabilitato la funzione analogica da ANSELA, ANSELB...)
 */

#include "conf_16f887.h"
#define _XTAL_FREQ 20000000
#include <xc.h>

#define S10 PORTAbits.RA0  // USCITA SEGNALE S1dI AL ROSSO
#define S11 PORTAbits.RA1  // USCITA SEGNALE S1dI AL VERDE
#define S12 PORTAbits.RA2  // USCITA SEGNALE S1dII AL VERDE
#define S30 PORTAbits.RA3  // USCITA SEGNALE S3d AL ROSSO
#define S31 PORTAbits.RA4  // USCITA SEGNALE S3d AL VERDE
#define S40 PORTAbits.RA5  // USCITA SEGNALE S4d AL ROSSO
#define S41 PORTBbits.RB0  // USCITA SEGNALE S4d AL VERDE
#define S32 PORTBbits.RB1  // USCITA SEGNALE S3s AL ROSSO
#define S33 PORTBbits.RB2  // USCITA SEGNALE S3s AL VERDE
#define S20 PORTBbits.RB3  // USCITA SEGNALE S2sI AL ROSSO
#define S21 PORTBbits.RB4  // USCITA SEGNALE S2sI AL VERDE
#define S22 PORTBbits.RB5  // USCITA SEGNALE S2sII AL VERDE
#define S42 PORTBbits.RB6  // USCITA SEGNALE S4s AL ROSSO
#define S43 PORTBbits.RB7  // USCITA SEGNALE S4s AL VERDE
#define B10 PORTCbits.RC0  // INGRESSO CDB 10
#define B11 PORTCbits.RC1  // INGRESSO CDB 11
#define B12 PORTCbits.RC2  // INGRESSO CDB 12
#define B0I PORTCbits.RC3  // INGRESSO CDB I
#define BII PORTCbits.RC4  // INGRESSO CDB II
#define B22 PORTCbits.RC5  // INGRESSO CDB 22
#define B21 PORTCbits.RC6  // INGRESSO CDB 21
#define B20 PORTCbits.RC7  // INGRESSO CDB 20
#define CD1 PORTDbits.RD0  // INGRESSO PULSANTE COMANDO DEVIATOIO 1 N/R
#define CD2 PORTDbits.RD1  // INGRESSO PULSANTE COMANDO DEVIATOIO 2 N/R
#define D1N PORTDbits.RD2  // USCITA CONTROLLO DEVIATOIO 1N
#define D1R PORTDbits.RD3  // USCITA CONTROLLO DEVIATOIO 1R
#define D2N PORTDbits.RD4  // USCITA CONTROLLO DEVIATOIO 2N
#define D2R PORTDbits.RD5  // USCITA CONTROLLO DEVIATOIO 2R
#define I13 PORTDbits.RD6; // INGRESSO PULSANTE ITINERARIO 1-3
#define I14 PORTDbits.RD7; // INGRESSO PULSANTE ITINERARIO 1-4
#define I23 PORTEbits.RE0; // INGRESSO PULSANTE ITINERARIO 2-3
#define I24 PORTEbits.RE1; // INGRESSO PULSANTE ITINERARIO 2-4
#define CBA PORTEbits.RE2; // USCITA CONTROLLO CDB LIBERI


// prototipi funzione
void settings(void);//va messo fuori dal main

int main(void)
    {
    // imposto i registri tristato 
    TRISA=0;            // PORTA TUTTA IN USCITA PER I SEGNALI
    TRISB=0;            // PORTB TUTTA IN USCITA PER I SEGNALI
    TRISC=1;            // PORTC TUTTI INGRESSI CDB
    TRISD=0b11000011;   // PORTD0-RD1 PULS. COMANDO DEV 1 e 2 RD2-RD5 USCITE LED KD1 e KD2 N/R - RD6-RD7 PULS. ITINERARIO 1-3 e 1-4
    TRISE=0b011;        // PORTE0-RE1 PULSANTI DI ITINERARIO 2-3 e 2-4 RE2 CONTROLLO TUTTI CDB LIBERI
    
    settings();
    
    while(1)
        {
        if (!CD1)
            {
                // eseguo un controllo antibounce
             __delay_ms(100);
            if (!CD1)
                {
                D1N^=1;
                D1R^=1;
                // rimango bloccato fino a che non viene rilasciato il pulsante
                while(!CD1);
                // attendo un po' prima di tornare al main per non calcolare
                // i rimbalzi dovuti al rilascio
                __delay_ms(100);
                }
            } //QUI SI CHIUDE IL CODICE PER IL DEVIATOIO N.1
        if (!CD2)
            {
                // eseguo un controllo antibounce
             __delay_ms(100);
            if (!CD2)
                {
                D2N^=1;
                D2R^=1;
                // rimango bloccato fino a che non viene rilasciato il pulsante
                while(!CD2);
                // attendo un po' prima di tornare al main per non calcolare
                // i rimbalzi dovuti al rilascio
                __delay_ms(100);
                }
            } //QUI SI CHIUDE IL CODICE PER IL DEVIATOIO N.2
        
        
        
        if (PORTCbits.RC0==0)
            {
            S43=0;// Led controllo CDB
            }
        else
    {
      S43=1;
        /*if (PORTCbits.RC1==0)
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
        if (PORTCbits.RC2==0)
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
        if (PORTCbits.RC3==0)
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
        if (PORTCbits.RC4==0)
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
        if (PORTCbits.RC5==0)
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
        if (PORTCbits.RC6==0)
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
        if (PORTCbits.RC7==0)                                        
            {
            S43=0;// Led controllo CDB
            }
      else
    {
      S43=1;
         */
            }//QUI SI CHIUDE IL CODICE DI PROVA TEST INGRESSI CDB TUTTI ALTI
      
        }// fine ciclo infinito
    }// fine main
        
    
        
        


void settings(void)
    {
    ANSEL=0; // pins A/D (0-7) come digitali
    ANSELH=0; // pins A/D (8-13) come digitali
    ADCON0bits.ADON = 0;        //modulo AD spento

    
    
    // Led controllo deviatoi
    D1N=1;
    D1R=0;
    D2N=1;
    D2R=0;
    
    
    
    //Led accensione segnali
    S10=1;
    S11=0;
    S12=0;
    S30=1;
    S31=0;
    S40=1;
    S41=0;
    S32=1;
    S33=0;
    S20=1;
    S21=0;
    S22=0;
    S42=1;
    S43=0;
    
}